# coding: utf-8

# flake8: noqa

"""
    Klines Service API

         API for retrieving OHLCV data, funding rates, and symbol information from Binance.          ## WebSocket Support     Connect to `/ws` to receive real-time OHLCV updates.          Example subscription message:     ```json     {         \"action\": \"subscribe\",         \"market\": \"spot\",         \"symbol\": \"BTCUSDT\",         \"timeframe\": \"15m\"     }     ```     

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


__version__ = "1.0.0"

# import apis into sdk package
from crypticorn.klines.client.api.funding_rates_api import FundingRatesApi
from crypticorn.klines.client.api.health_check_api import HealthCheckApi
from crypticorn.klines.client.api.ohlcv_data_api import OHLCVDataApi
from crypticorn.klines.client.api.symbols_api import SymbolsApi
from crypticorn.klines.client.api.udf_api import UDFApi

# import ApiClient
from crypticorn.klines.client.api_response import ApiResponse
from crypticorn.klines.client.api_client import ApiClient
from crypticorn.klines.client.configuration import Configuration
from crypticorn.klines.client.exceptions import OpenApiException
from crypticorn.klines.client.exceptions import ApiTypeError
from crypticorn.klines.client.exceptions import ApiValueError
from crypticorn.klines.client.exceptions import ApiKeyError
from crypticorn.klines.client.exceptions import ApiAttributeError
from crypticorn.klines.client.exceptions import ApiException

# import models into sdk package
from crypticorn.klines.client.models.base_response_health_check_response import (
    BaseResponseHealthCheckResponse,
)
from crypticorn.klines.client.models.base_response_list_funding_rate_response import (
    BaseResponseListFundingRateResponse,
)
from crypticorn.klines.client.models.base_response_list_str import BaseResponseListStr
from crypticorn.klines.client.models.base_response_ohlcv_response import (
    BaseResponseOHLCVResponse,
)
from crypticorn.klines.client.models.error_response import ErrorResponse
from crypticorn.klines.client.models.exchange import Exchange
from crypticorn.klines.client.models.funding_rate_response import FundingRateResponse
from crypticorn.klines.client.models.http_validation_error import HTTPValidationError
from crypticorn.klines.client.models.health_check_response import HealthCheckResponse
from crypticorn.klines.client.models.history_error_response import HistoryErrorResponse
from crypticorn.klines.client.models.history_no_data_response import (
    HistoryNoDataResponse,
)
from crypticorn.klines.client.models.history_success_response import (
    HistorySuccessResponse,
)
from crypticorn.klines.client.models.market import Market
from crypticorn.klines.client.models.ohlcv_response import OHLCVResponse
from crypticorn.klines.client.models.resolution import Resolution
from crypticorn.klines.client.models.response_get_history_udf_history_get import (
    ResponseGetHistoryUdfHistoryGet,
)
from crypticorn.klines.client.models.search_symbol_response import SearchSymbolResponse
from crypticorn.klines.client.models.sort_direction import SortDirection
from crypticorn.klines.client.models.symbol_group_response import SymbolGroupResponse
from crypticorn.klines.client.models.symbol_info_response import SymbolInfoResponse
from crypticorn.klines.client.models.symbol_type import SymbolType
from crypticorn.klines.client.models.timeframe import Timeframe
from crypticorn.klines.client.models.udf_config_response import UDFConfigResponse
from crypticorn.klines.client.models.validation_error import ValidationError
from crypticorn.klines.client.models.validation_error_loc_inner import (
    ValidationErrorLocInner,
)
