# coding: utf-8

"""
    Hive AI API

    API for Hive AI model training and evaluation

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List
from crypticorn.hive.client.models.coins import Coins
from crypticorn.hive.client.models.evaluation import Evaluation
from crypticorn.hive.client.models.model_status import ModelStatus
from crypticorn.hive.client.models.target import Target
from crypticorn.hive.client.models.target_type import TargetType
from typing import Optional, Set
from typing_extensions import Self


class Model(BaseModel):
    """
    Pydantic model for model response
    """  # noqa: E501

    model_id: StrictInt = Field(description="Unique model identifier")
    name: StrictStr = Field(description="Model name")
    coin_id: Coins = Field(description="Coin ID")
    target: Target = Field(description="Target variable")
    status: ModelStatus = Field(description="Model status")
    target_type: TargetType = Field(description="Target type")
    evaluations: List[Evaluation] = Field(description="Model evaluations")
    user_id: StrictStr = Field(description="Developer user ID")
    created_at: datetime = Field(description="Model creation timestamp")
    updated_at: datetime = Field(description="Model update timestamp")
    __properties: ClassVar[List[str]] = [
        "model_id",
        "name",
        "coin_id",
        "target",
        "status",
        "target_type",
        "evaluations",
        "user_id",
        "created_at",
        "updated_at",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Model from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in evaluations (list)
        _items = []
        if self.evaluations:
            for _item_evaluations in self.evaluations:
                if _item_evaluations:
                    _items.append(_item_evaluations.to_dict())
            _dict["evaluations"] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Model from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "model_id": obj.get("model_id"),
                "name": obj.get("name"),
                "coin_id": obj.get("coin_id"),
                "target": obj.get("target"),
                "status": obj.get("status"),
                "target_type": obj.get("target_type"),
                "evaluations": (
                    [Evaluation.from_dict(_item) for _item in obj["evaluations"]]
                    if obj.get("evaluations") is not None
                    else None
                ),
                "user_id": obj.get("user_id"),
                "created_at": obj.get("created_at"),
                "updated_at": obj.get("updated_at"),
            }
        )
        return _obj
