# coding: utf-8

"""
    Klines Service API

         API for retrieving OHLCV data, funding rates, and symbol information from Binance.          ## WebSocket Support     Connect to `/ws` to receive real-time OHLCV updates.          Example subscription message:     ```json     {         \"action\": \"subscribe\",         \"market\": \"spot\",         \"symbol\": \"BTCUSDT\",         \"timeframe\": \"15m\"     }     ```     

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class Resolution(str, Enum):
    """
    Resolution
    """

    """
    allowed enum values
    """
    ENUM_15 = "15"
    ENUM_30 = "30"
    ENUM_60 = "60"
    ENUM_240 = "240"
    ENUM_1D = "1D"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of Resolution from a JSON string"""
        return cls(json.loads(json_str))
